<!DOCTYPE html>
<html lang="en">
  <head>

    <meta charset="utf-8" />

    <title>This site requires cookies</title>

</head>
<body>

<h1>This site requires cookies</h1>

<p>Your browser doesn't accept cookies, but this site needs them to work.</p>

<p>Don't worry, it's easy to fix. <a href="http://support.google.com/accounts/bin/answer.py?hl=en&answer=61416" target="_blank">Lean how to accept cookies</a>.</p>

<p>Once you got cookies turn on, you can <a href="{{ next }}">go back to the site.</a></p>

</body>
</html>
